// SPDX-FileCopyrightText: 2023 Pieter Hijma <pieterhijma@noreply.codeberg.org>
//
// SPDX-License-Identifier: GPL-3.0-or-later

#define FRONT

#ifdef FRONT
#define PIN_IR_LED    D1
#define PIN_IR_SENSOR D0

#else

#define PIN_IR_LED    D3
#define PIN_IR_SENSOR D2
#endif


#define NR_SAMPLES  100

#define NR_SAMPLES_AVG 50

#define DETECTION_FACTOR 0.2

void setup() {
   pinMode(PIN_IR_LED, OUTPUT);
	 pinMode(PIN_IR_SENSOR, INPUT);
   Serial.begin(9600);
	 delay(3000);
   Serial.println("starting up");
}

int messagedReady = 0;

int nrSamplesAvg = 0;
int sumAvg = 0;

int nrCarsDetected = 0;

void loop() {
	int valLED = 0;
	int valAmbient = 0;

	for (int i = 0; i < NR_SAMPLES; i++) {
		digitalWrite(PIN_IR_LED, HIGH);
		delay(1);
		valLED += analogRead(PIN_IR_SENSOR);
		
		digitalWrite(PIN_IR_LED, LOW);
		delay(1);
		valAmbient += analogRead(PIN_IR_SENSOR);
	}

	// @waag
	// if the LED is off, valLow increases from 3850 to 4096 with something closer
	// if the LED is on, valHigh decreases from 250 to 230 with something closer
	//                                          3600 to 3870 = 270

	// @home
	// if the LED is off, valLow increases from 310 to 380 with something closer
	// if the LED is on, valHigh decreases from 220 to 217 maybe
	
	int diff = (valAmbient - valLED) / NR_SAMPLES;
	
	Serial.print("ambient: ");
	Serial.println(valAmbient / NR_SAMPLES);
	Serial.print("led: ");
	Serial.println(valLED / NR_SAMPLES);
	Serial.print("diff: ");
	Serial.println(diff);
	
	if (nrSamplesAvg == NR_SAMPLES_AVG) {
		// we are ready to detect now
		if (!messagedReady) {
			Serial.println("Ready to detect cars");
			messagedReady = 1;
		}
		
		float previousAvg = sumAvg / (float) nrSamplesAvg;
		Serial.print("avg: ");
		Serial.println(previousAvg);
		Serial.println("");
		
		if (diff > previousAvg * (1 + DETECTION_FACTOR) || diff < previousAvg * (1 - DETECTION_FACTOR)) {
			nrCarsDetected++;
			Serial.print("Car ");
			Serial.println(nrCarsDetected);
			delay(2000);
			// we don't count this in the average
		}
		else {
			// replace the previous average with the current value
			sumAvg -= previousAvg;
			sumAvg += diff;
		}
	}
	else {
		// we are collecting samples
		nrSamplesAvg++;
		sumAvg += diff;
	}
	
}
