# SPDX-FileCopyrightText: 2023 Pieter Hijma <pieterhijma@noreply.codeberg.org>
#
# SPDX-License-Identifier: LGPL-3.0-or-later

import math

import FreeCAD as App
import Part

from util import *

import LegPillar as leg

TOP = True
BOTTOM = False
DEFAULT_LEVEL = 0

# def isLegPillar(obj):
#     return 'Proxy' in obj.PropertiesList and type(obj.Proxy).__name__ == 'BracketPillar'

# def isLegPillarWithLevel(obj, level):
#     return isLegPillar(obj) and obj.Level == level

def computeEdgesSlot(pF, pL, top):
    x1 = leg.WIDTH_GAP / 2 if top else -leg.WIDTH_GAP / 2
    y1 = pF.y

    x2 = x1
    y2 = y1 + (leg.WIDTH_LEG/2 if top else -leg.WIDTH_LEG/2)

    p1 = mkP(x1, y1)
    p2 = mkP(x2, y2)
    p3 = mirrorP(p2)
    p4 = mirrorP(p1)

    return [mkE(pF, p1),
            mkE(p1, p2),
            mkE(p2, p3),
            mkE(p3, p4),
            mkE(p4, pL)]

def getXTR(obj, level):
    print(f"computing for level {level}")
    if level == 0:
        # checked, seems right
        return leg.WIDTH_TOP/2 + leg.THICKNESS_WOOD
    else:
        pillar = obj.InList[0]
        nrHoles = leg.getNrHoles(pillar)
        i = nrHoles - level + 1
        print(f"the i: {i}")
        print(f"leg.widthBase: {leg.widthBase(pillar.Height)}")
        # checked, seems right
        res = (leg.widthBase(pillar.Height)/2 -
               i * math.cos(leg.ANGLE_SIDE) * leg.DEFAULT_HEIGHT +
               leg.THICKNESS_WOOD)
        print(f"the result: {res}")
        return res
    

def computeShape(obj):
    level = obj.Level
    top = obj.Top

    xTR = getXTR(obj, level)
    yTR = 0

    xBR = xTR + leg.WIDTH_LEG / math.tan(leg.ANGLE_SIDE)
    yBR = -leg.WIDTH_LEG

    # checked is right
    xBRO = xBR - leg.THICKNESS_WOOD
    yBRO = yBR

    xTRO = xBRO - (leg.WIDTH_LEG/2) / math.tan(leg.ANGLE_SIDE)
    yTRO = -leg.WIDTH_LEG/2

    # checked, seems right
    xTRI = xTRO - math.sin(leg.ANGLE_SIDE) * leg.WIDTH_GAP
    yTRI = yTRO

    xBRI = xTRI
    yBRI = -leg.WIDTH_LEG

    # points
    pTR = mkP(xTR, yTR)
    pBR = mkP(xBR, yBR)

    pBRO = mkP(xBRO, yBRO)
    pTRO = mkP(xTRO, yTRO)

    pTRI = mkP(xTRI, yTRI)
    pBRI = mkP(xBRI, yBRI)

    pBLI = mirrorP(pBRI)
    pTLI = mirrorP(pTRI)

    pTLO = mirrorP(pTRO)
    pBLO = mirrorP(pBRO)

    pBL = mirrorP(pBR)
    pTL = mirrorP(pTR)

    #edges
    edges = []
    edges.append(mkE(pTR, pBR))
    edges.append(mkE(pBR, pBRO))
    edges.append(mkE(pBRO, pTRO))
    edges.append(mkE(pTRO, pTRI))
    edges.append(mkE(pTRI, pBRI))
    if top:
        edges.extend(computeEdgesSlot(pBRI, pBLI, top))
    else:
        edges.append(mkE(pBRI, pBLI))
    edges.append(mkE(pBLI, pTLI))
    edges.append(mkE(pTLI, pTLO))
    edges.append(mkE(pTLO, pBLO))
    edges.append(mkE(pBLO, pBL))
    edges.append(mkE(pBL, pTL))
    if top:
        edges.append(mkE(pTL, pTR))
    else:
        edges.extend(computeEdgesSlot(pTL, pTR, top))

    # wires
    w = mkW(edges)

    # faces
    f = Part.Face(w)

    # solids
    # checked, is right
    e = f.extrude(App.Vector(0, 0, leg.THICKNESS_WOOD))
    
    obj.Shape = e
    

def createBracketPillar():
    doc = App.ActiveDocument
    obj = doc.addObject("Part::FeaturePython", "BracketPillar")
    BracketPillar(obj)
    obj.ViewObject.Proxy = 0 # This is mandatory unless we code the ViewProvider too.
    return obj


class BracketPillar:

    def __init__(self, obj):
        obj.Proxy = self
        obj.addProperty("App::PropertyInteger", "Level").Level = DEFAULT_LEVEL
        obj.addProperty("App::PropertyBool", "Top").Top = TOP
        obj.addExtension('Part::AttachExtensionPython')

    def onChanged(self, obj, prop):
        ''' Do something when a property has changed '''
        print(f"BracketPillar.onChanged: {prop}")
        pl = obj.PropertiesList
        if 'Level' in pl and 'Top' in pl and (
                prop == 'Level' or prop == 'Top'):
            computeShape(obj)

    def execute(self, obj):
        ''' Do something when doing a recomputation '''
        pass

                        
        
