# SPDX-FileCopyrightText: 2023 Pieter Hijma <pieterhijma@noreply.codeberg.org>
#
# SPDX-License-Identifier: LGPL-3.0-or-later

import math

import FreeCAD as App
import Part

from util import *

import LegPillar as leg


def computeEdgesSlot(pF, pL, top):
    x1 = leg.THICKNESS_WOOD / 2 if top else -leg.THICKNESS_WOOD / 2
    y1 = pF.y

    x2 = x1
    y2 = y1 + (leg.WIDTH_LEG/2 if top else -leg.WIDTH_LEG/2)

    p1 = mkP(x1, y1)
    p2 = mkP(x2, y2)
    p3 = mirrorP(p2)
    p4 = mirrorP(p1)

    return [mkE(pF, p1),
            mkE(p1, p2),
            mkE(p2, p3),
            mkE(p3, p4),
            mkE(p4, pL)]


def computeShape(obj):
    # checked, seems right
    xTR = leg.WIDTH_TOP/2 + leg.THICKNESS_WOOD
    yTR = 0

    xBR = xTR + leg.WIDTH_LEG / math.tan(leg.ANGLE_SIDE)
    yBR = -leg.WIDTH_LEG

    # checked, seems right
    xBRO = xBR - leg.THICKNESS_WOOD
    yBRO = yBR

    xTRO = xBRO - (leg.WIDTH_LEG/2) / math.tan(leg.ANGLE_SIDE)
    yTRO = -leg.WIDTH_LEG/2

    xTRI = xTRO - math.sin(leg.ANGLE_SIDE) * leg.THICKNESS_WOOD
    yTRI = yTRO

    xBRI = xTRI
    yBRI = -leg.WIDTH_LEG

    # points
    pTR = mkP(xTR, yTR)
    pBR = mkP(xBR, yBR)

    pBRO = mkP(xBRO, yBRO)
    pTRO = mkP(xTRO, yTRO)

    pTRI = mkP(xTRI, yTRI)
    pBRI = mkP(xBRI, yBRI)

    pBLI = mirrorP(pBRI)
    pTLI = mirrorP(pTRI)

    pTLO = mirrorP(pTRO)
    pBLO = mirrorP(pBRO)

    pBL = mirrorP(pBR)
    pTL = mirrorP(pTR)

    # checked, seems right
    xSLL = -leg.WIDTH_TRACK/2 - leg.WIDTH_GAP
    pSTLL = mkP(xSLL, 0)
    pSBLL = mkP(xSLL, -leg.HEIGHT_SLOT)
    xSRL = -leg.WIDTH_TRACK/2
    pSBRL = mkP(xSRL, -leg.HEIGHT_SLOT)
    pSTRL = mkP(xSRL, 0)

    pSTLR = mirrorP(pSTRL)
    pSBLR = mirrorP(pSBRL)
    pSBRR = mirrorP(pSBLL)
    pSTRR = mirrorP(pSTLL)
    
    #edges
    edges = [mkE(pTR, pBR),
             mkE(pBR, pBRO),
             mkE(pBRO, pTRO),
             mkE(pTRO, pTRI),
             mkE(pTRI, pBRI),
             mkE(pBRI, pBLI),
             mkE(pBLI, pTLI),
             mkE(pTLI, pTLO),
             mkE(pTLO, pBLO),
             mkE(pBLO, pBL),
             mkE(pBL, pTL),
             mkE(pTL, pSTLL),
             mkE(pSTLL, pSBLL),
             mkE(pSBLL, pSBRL),
             mkE(pSBRL, pSTRL),
             mkE(pSTRL, pSTLR),
             mkE(pSTLR, pSBLR),
             mkE(pSBLR, pSBRR),
             mkE(pSBRR, pSTRR),
             mkE(pSTRR, pTR)]

    # wires
    w = mkW(edges)

    # faces
    f = Part.Face(w)

    # solids
    # checked, is right
    e = f.extrude(App.Vector(0, 0, leg.THICKNESS_WOOD))
    
    obj.Shape = e
    

def createTopBracketPillar():
    doc = App.ActiveDocument
    obj = doc.addObject("Part::FeaturePython", "TopBracketPillar")
    TopBracketPillar(obj)
    obj.ViewObject.Proxy = 0 # This is mandatory unless we code the ViewProvider too.
    return obj


class TopBracketPillar:

    def __init__(self, obj):
        obj.Proxy = self
        #obj.addProperty("App::PropertyBool", "Top").Top = TOP
        obj.addExtension('Part::AttachExtensionPython')
        computeShape(obj)

    def onChanged(self, obj, prop):
        ''' Do something when a property has changed '''
        # print(f"TopBracketPillar.onChanged: {prop}")
        # pl = obj.PropertiesList
        # if 'Level' in pl and 'Top' in pl and (
        #         prop == 'Level' or prop == 'Top'):
        #     computeShape(obj)
        pass

    def execute(self, obj):
        ''' Do something when doing a recomputation '''
        pass

                        
        
